/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ForecastFox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gSearch = null;

function searchLoad()
{
  gSearch = new SearchModule();
  gSearch.init();
}

function searchUnload()
{
  gSearch.destroy();
  gSearch = null;
}

function SearchModule()
{
  this.wrappedJSObject = this;
  return this;
}

SearchModule.prototype = {
  mUpdate: null,
  mRequest: null,
  mBundle: null,
  mText: null,
  mResults: null,
  mRunning: false,
  mBaseURL: "http://xoap.weather.com/search/search?where=",
       
  init: function()
  {
    this.mUpdate  = Components.classes["@mozdev.org/forecastfox/update;1"].getService(Components.interfaces.nsIForecastfoxUpdate); 
    this.mBundle  = document.getElementById("ff-bundle-search");
    this.mText = document.getElementById("ff-text-search");
    this.mText.focus();
    this.mText.select();
    this.mResults = document.getElementById("ff-items-results");
  },
  
  destroy: function()
  {
    this.mUpdate = null;
    this.mRunning = null;
    this.mRequest = null;
    this.mBundle = null;
    this.mBaseURL = null;
    this.mText = null;
    this.mResults = null;
  },
  
  run: function()
  {
    //remove old search
    this.clear();
    
    //do nothing if string not in textbox or request running
    if (!this.mText.value || this.mRunning)
      return;  
        
    //set running
    this.mRunning = true;

    //set label
    this.setLabel("ff.search.loading", null);

    //set url
    var URL = encodeURI(this.mBaseURL + this.mText.value);

    //setup request and send
    var module = this;
    this.mRequest = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);
    this.mRequest.open("GET", URL, true);
    this.mRequest.overrideMimeType("text/xml");
    this.mRequest.onload = function() { module.onSearchLoad(this.responseXML, this.status) };
    this.mRequest.onerror = function() { module.onSearchError() };
    this.mRequest.send(null);          
  },
  
  clear: function()
  {
    while (this.mResults.hasChildNodes())
      this.mResults.removeChild(this.mResults.lastChild);    
  },
  
  setLabel: function(aText, aNode)
  {
    var treeitem, treerow, treecell1, treecell2, treecell3;  

    treeitem = document.createElement("treeitem");
    treerow = document.createElement("treerow");
    treecell1 = document.createElement("treecell");
    treecell2 = document.createElement("treecell");  
    treecell3 = document.createElement("treecell");
          
    if (aText) {
      treeitem.setAttribute("id", "na");  
      treecell1.setAttribute("label", this.mBundle.getString(aText));
      treecell2.setAttribute("label", "");
      treecell3.setAttribute("label", "");  
    } else {

      treeitem.setAttribute("id", aNode.getAttribute("id"));  
      treecell1.setAttribute("label", aNode.childNodes[0].nodeValue);
      treecell2.setAttribute("label", aNode.getAttribute("id"));
      treecell3.setAttribute("label", aNode.getAttribute("type"));     
    }
    
    treerow.appendChild(treecell1);
    treerow.appendChild(treecell2);  
    treerow.appendChild(treecell3);
    treeitem.appendChild(treerow);
    this.mResults.appendChild(treeitem);
  },
  
  onSearchLoad: function(aResponse, aStatus)
  {
    var treeitem, treerow, treecell1, treecell2, treecell3;   
  
    //set running to false
    this.mRunning = false;
  
    //clear old results
    this.clear();
  
    //response error
    if (!aResponse || aStatus > 200) {
      this.setLabel("ff.search.error", null);
      return;
    }

    //no locations
    var locs = aResponse.getElementsByTagName("loc");
    if (locs.length == 0) {
      this.setLabel("ff.search.location", null);
      return;
    }

    //populate tree
    for (var i=0; i < locs.length; i++)
      this.setLabel(null, locs[i]);
  },
  
  onSearchError: function()
  {
    //set running to false
    this.mRunning = false;
  
    //clear old results
    this.clear();
  

    //set error in tree
    this.setLabel("ff.search.error", null);  
  },
  
  accept: function(aClose)
  {
    var tree = document.getElementById("ff-tree-results");
  
    //if nothing selected return
    if (tree.currentIndex == -1) {
      if (aClose) {
        window.close();
        return;
      } else
      return;
    }
  
    //if no location return
    var current = tree.view.getItemAtIndex(tree.currentIndex).getAttribute("id");
    if (current == "na") {
      if (aClose) {
        window.close();
        return;
      } else
      return;
    }

    //set location value
    var locid = window.opener.document.getElementById("ff-text-code");
    locid.value = current;
  
    //update apply button
    window.opener.updateButtons();
    window.close();  
  },
  
  openLink: function(aEvent, aTWC)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aTWC)
      url = url + "?prod=" + this.mUpdate.productCode + "&par=" + this.mUpdate.partnerId;

    window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
  }
}